use std::env;
use std::fs;

pub fn parse_input() -> String {
    let args: Vec<String> = env::args().collect();
    let input_path = args
        .get(1)
        .expect(format!("Usage: {} INPUT-FILE", args.get(0).unwrap()).as_str());

    fs::read_to_string(input_path)
        .expect(format!("Couldn't read input file {}", input_path).as_str())
}

pub fn max_n<T: Ord + Copy>(slice: &[T], n: usize) -> Result<Vec<T>, ()> {
    if n == 0 || n > slice.len() {
        return Err(());
    }
    let mut max_vals = Vec::new();
    for _ in 0..n {
        max_vals.push(slice[0]);
    }

    for val in slice {
        if val > &max_vals[0] {
            max_vals[2] = max_vals[1];
            max_vals[1] = max_vals[0];
            max_vals[0] = *val;
        } else if val > &max_vals[1] {
            max_vals[2] = max_vals[1];
            max_vals[1] = *val;
        } else if val > &max_vals[2] {
            max_vals[2] = *val;
        }
    }

    Ok(max_vals)
}

pub type Coord = (usize, usize);

pub trait Coordinate {
    fn manhattan(&self, other: &Self) -> usize;
    fn to_signed(&self) -> SignedCoord;
    fn to_unsigned(&self) -> Coord;
}

impl Coordinate for Coord {
    fn manhattan(&self, other: &Self) -> usize {
        return self.0.abs_diff(other.0) + self.1.abs_diff(other.1);
    }

    fn to_signed(&self) -> SignedCoord {
        (self.0 as isize, self.1 as isize)
    }

    fn to_unsigned(&self) -> Coord {
        *self
    }
}

pub type SignedCoord = (isize, isize);

impl Coordinate for SignedCoord {
    fn manhattan(&self, other: &Self) -> usize {
        return self.0.abs_diff(other.0) + self.1.abs_diff(other.1);
    }

    fn to_signed(&self) -> SignedCoord {
        *self
    }

    fn to_unsigned(&self) -> Coord {
        (self.0 as usize, self.1 as usize)
    }
}
